package com.qmth.paper.library.business.bean.params;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import io.swagger.annotations.ApiModelProperty;
import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.Range;

import javax.validation.constraints.NotNull;
import java.io.Serializable;

/**
 * @Description: 扫描任务参数
 * @Author: CaoZixuan
 * @Date: 2022-09-28
 */
public class PaperScanTaskParam implements Serializable {

    @ApiModelProperty(value = "扫描任务id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long paperScanTaskId;

    @ApiModelProperty(value = "扫描任务名称")
    @NotNull(message = "请输入扫描任务名称")
    @Length(min = 1, message = "请输入扫描任务名称")
    private String scanTaskName;

    @ApiModelProperty(value = "档案id")
    @JsonSerialize(using = ToStringSerializer.class)
    @NotNull(message = "请选择档案")
    @Range(min = 1L, message = "请选择档案")
    private Long paperArchivesId;

    @ApiModelProperty(value = "课程代码")
    @NotNull(message = "请输入课程代码")
    @Length(min = 1, message = "请输入课程代码")
    private String courseCode;

    @ApiModelProperty(value = "课程名称")
    @NotNull(message = "请输入课程名称")
    @Length(min = 1, message = "请输入课程名称")
    private String courseName;

    @ApiModelProperty(value = "扫描员")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long scanUserId;

    @ApiModelProperty(value = "扫描份数（学生数）")
    private Integer studentCount;

    public Long getPaperScanTaskId() {
        return paperScanTaskId;
    }

    public void setPaperScanTaskId(Long paperScanTaskId) {
        this.paperScanTaskId = paperScanTaskId;
    }

    public String getScanTaskName() {
        return scanTaskName;
    }

    public void setScanTaskName(String scanTaskName) {
        this.scanTaskName = scanTaskName;
    }

    public Long getPaperArchivesId() {
        return paperArchivesId;
    }

    public void setPaperArchivesId(Long paperArchivesId) {
        this.paperArchivesId = paperArchivesId;
    }

    public String getCourseCode() {
        return courseCode;
    }

    public void setCourseCode(String courseCode) {
        this.courseCode = courseCode;
    }

    public String getCourseName() {
        return courseName;
    }

    public void setCourseName(String courseName) {
        this.courseName = courseName;
    }

    public Long getScanUserId() {
        return scanUserId;
    }

    public void setScanUserId(Long scanUserId) {
        this.scanUserId = scanUserId;
    }

    public Integer getStudentCount() {
        return studentCount;
    }

    public void setStudentCount(Integer studentCount) {
        this.studentCount = studentCount;
    }
}
