package com.qmth.paper.library.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.paper.library.business.bean.result.PaperLibraryResult;
import com.qmth.paper.library.business.bean.result.PaperScanTaskResult;
import com.qmth.paper.library.business.bean.result.TaskStudentResult;
import com.qmth.paper.library.business.entity.PaperLibrary;
import com.qmth.paper.library.business.entity.PaperScanTask;
import com.qmth.paper.library.business.entity.PaperScanTaskDetail;
import com.qmth.paper.library.business.mapper.PaperLibraryMapper;
import com.qmth.paper.library.business.service.PaperLibraryCommonService;
import com.qmth.paper.library.business.service.PaperLibraryService;
import com.qmth.paper.library.business.service.PaperScanTaskDetailService;
import com.qmth.paper.library.business.service.PaperScanTaskService;
import com.qmth.paper.library.common.contant.SystemConstant;
import com.qmth.paper.library.common.entity.SysUser;
import com.qmth.paper.library.common.enums.ExceptionResultEnum;
import com.qmth.paper.library.common.enums.RecognitionTypeEnum;
import com.qmth.paper.library.common.enums.RoleTypeEnum;
import com.qmth.paper.library.common.enums.UploadFileEnum;
import com.qmth.paper.library.common.lock.LockService;
import com.qmth.paper.library.common.lock.LockType;
import com.qmth.paper.library.common.service.BasicStudentService;
import com.qmth.paper.library.common.service.SysUserService;
import com.qmth.paper.library.common.util.*;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONArray;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * <p>
 * 图片库 服务实现类
 * </p>
 */
@Service
public class PaperLibraryServiceImpl extends ServiceImpl<PaperLibraryMapper, PaperLibrary> implements PaperLibraryService {

    @Resource
    PaperScanTaskService paperScanTaskService;

    @Resource
    PaperLibraryService paperLibraryService;

    @Resource
    PaperScanTaskDetailService paperScanTaskDetailService;

    @Resource
    PaperLibraryCommonService paperLibraryCommonService;

    @Resource
    SysUserService sysUserService;

    @Resource
    BasicStudentService basicStudentService;

    @Resource
    OcrUtil ocrUtil;

    @Resource
    LockService lockService;

    @Resource
    FileStoreUtil fileStoreUtil;

    @Override
    public IPage<PaperScanTaskResult> pageUnbindData(Long semesterId, Long paperArchivesId, String courseCode, String scanTaskName, Integer pageNumber, Integer pageSize) {
        return paperScanTaskService.pageUnbindData(new Page<>(pageNumber, pageSize), semesterId, paperArchivesId, courseCode, scanTaskName);
    }

    @Override
    public IPage<PaperLibraryResult> pageBindData(Long semesterId, Long paperArchivesId, String courseCode, String scanTaskName, Integer pageNumber, Integer pageSize) {
        return this.baseMapper.pageBindData(new Page<>(pageNumber, pageSize), semesterId, paperArchivesId, courseCode, scanTaskName);
    }

    @Override
    public int countBindData(Long paperScanTaskId) {
        return this.baseMapper.countBindData(paperScanTaskId);
    }

    @Override
    public IPage<PaperLibraryResult> pageAbnormalData(Long semesterId, Long paperArchivesId, String courseCode, String scanTaskName, Integer pageNumber, Integer pageSize) {
        Set<Long> listSubOrgIds = paperLibraryCommonService.listSubOrgIds(null);
        Long userId = Long.valueOf(ServletUtil.getRequestHeaderUserId().toString());
        boolean isMatch = sysUserService.checkRoleIsAppoint(RoleTypeEnum.DATA_CHECKER, userId);
        return this.baseMapper.pageAbnormalData(new Page<>(pageNumber, pageSize), semesterId, paperArchivesId, courseCode, scanTaskName, listSubOrgIds, isMatch, userId);
    }

    @Transactional
    @Override
    public boolean bind(Long paperLibraryId, Long paperScanTaskId, Long studentId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());

        PaperLibrary paperLibrary = this.getById(paperLibraryId);
        if (paperLibrary == null) {
            throw ExceptionResultEnum.ERROR.exception("绑定对象有误，图片已删除，请刷新数据再试");
        }

        PaperScanTaskDetail paperScanTaskDetail = paperScanTaskDetailService.getByPaperScanTaskIdAndStudentId(paperScanTaskId, studentId);
        if (paperScanTaskDetail == null) {
            paperScanTaskDetail = new PaperScanTaskDetail();
            paperScanTaskDetail.insertInfo(sysUser.getId());
            paperScanTaskDetail.setSchoolId(schoolId);
            paperScanTaskDetail.setPaperScanTaskId(paperScanTaskId);
            paperScanTaskDetail.setStudentId(studentId);
            paperScanTaskDetail.setBindCount(1);
            paperScanTaskDetailService.save(paperScanTaskDetail);
        }

        Boolean abnormal = paperLibrary.getAbnormal();
        UpdateWrapper<PaperLibrary> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(PaperLibrary::getPaperScanTaskDetailId, paperScanTaskDetail.getId())
                .set(PaperLibrary::getAbnormal, false)
                .eq(PaperLibrary::getId, paperLibraryId);
        this.update(updateWrapper);

        // 更新已绑定状态
        if (paperLibrary.getPaperScanTaskDetailId() != null) {
            PaperScanTaskDetail oldPaperScanTaskDetail = paperScanTaskDetailService.getById(paperLibrary.getPaperScanTaskDetailId());
            // 查询是否有绑定试卷
            int count = this.baseMapper.countByPaperScanTaskDetailId(paperLibrary.getPaperScanTaskDetailId());
            oldPaperScanTaskDetail.setBindCount(count);
            paperScanTaskDetailService.updateById(oldPaperScanTaskDetail);
        }
        int count = this.baseMapper.countByPaperScanTaskDetailId(paperScanTaskDetail.getId());
        paperScanTaskDetail.setUpdateTime(System.currentTimeMillis());
        paperScanTaskDetail.setBindCount(count);
        paperScanTaskDetailService.updateById(paperScanTaskDetail);

        // 任务下所有任务绑定完，更新状态
        PaperScanTask paperScanTask = paperScanTaskService.getById(paperScanTaskId);
        int studentCount = paperScanTaskDetailService.countStudent(paperScanTaskId);
        int bindCount = this.baseMapper.countBindData(paperLibrary.getPaperScanTaskId());
        // 总扫描数
        int scanCount = paperScanTask.getScanCount();
        Double progress = scanCount == 0 ? 0D :
                new BigDecimal(bindCount * 100).divide(new BigDecimal(scanCount), BigDecimal.ROUND_DOWN, 2).doubleValue();
        UpdateWrapper<PaperScanTask> taskUpdateWrapper = new UpdateWrapper<>();
        taskUpdateWrapper.lambda().set(PaperScanTask::getStudentCount, studentCount)
                .set(PaperScanTask::getBindCount, bindCount)
                .set(PaperScanTask::getProgress, progress)
                .set(PaperScanTask::getAbnormalCount, abnormal ? paperScanTask.getAbnormalCount() - 1 : paperScanTask.getAbnormalCount())
                .eq(PaperScanTask::getId, paperLibrary.getPaperScanTaskId());
        paperScanTaskService.update(taskUpdateWrapper);

        return true;
    }

    @Override
    public List<PaperLibraryResult> listStudentPicture(Long paperScanTaskDetailId) {
        Long schoolId = SystemConstant.convertIdToLong(String.valueOf(ServletUtil.getRequestHeaderSchoolId()));
        QueryWrapper<PaperLibrary> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(PaperLibrary::getSchoolId, schoolId)
                .eq(PaperLibrary::getPaperScanTaskDetailId, paperScanTaskDetailId)
                .orderByAsc(PaperLibrary::getSequence);
        List<PaperLibrary> paperLibraryList = this.list(queryWrapper);
        List<PaperLibraryResult> paperLibraryResultList = new ArrayList<>();
        for (PaperLibrary paperLibrary : paperLibraryList) {
            PaperLibraryResult paperLibraryResult = new PaperLibraryResult();
            BeanUtils.copyProperties(paperLibrary, paperLibraryResult);
            String fileUrl = paperLibrary.getPath();
            if (StringUtils.isBlank(fileUrl)) {
                throw ExceptionResultEnum.ERROR.exception("未找到图片路径");
            }
            List<JSONObject> objectList = JSON.parseArray(fileUrl, JSONObject.class);
            List<String> fileUrls = new ArrayList<>();
            for (JSONObject jsonObject : objectList) {
                fileUrls.add(paperLibraryCommonService.preViewPath(jsonObject.getString(SystemConstant.PATH), jsonObject.getString(SystemConstant.UPLOAD_TYPE), jsonObject.getString(SystemConstant.TYPE), false));
            }
            PaperScanTaskResult paperScanTaskResult = paperScanTaskService.getByPaperScanTaskId(paperLibrary.getPaperScanTaskId());
            if (paperScanTaskResult != null) {
                paperLibraryResult.setSemesterName(paperScanTaskResult.getSemesterName());
                paperLibraryResult.setPaperArchivesName(paperScanTaskResult.getArchivesName());
                paperLibraryResult.setPaperScanTaskName(paperScanTaskResult.getScanTaskName());
                paperLibraryResult.setCourseName(paperScanTaskResult.getCourseName());
//                paperLibraryResult.setTeacherName(paperScanTaskResult.getTeacherName());
//                paperLibraryResult.setTeachClazzName(paperScanTaskResult.getTeachClazzName());
            }
            paperLibraryResult.setFileUrls(fileUrls);
            paperLibraryResultList.add(paperLibraryResult);
        }
        return paperLibraryResultList;
    }

    @Override
    public List<PaperLibraryResult> toAbnormalPaper(Long paperLibraryId) {
        Long userId = Long.valueOf(ServletUtil.getRequestHeaderUserId().toString());
        Long schoolId = SystemConstant.convertIdToLong(String.valueOf(ServletUtil.getRequestHeaderSchoolId()));

        List<PaperLibraryResult> paperLibraryResultList = new ArrayList<>();
        PaperLibrary paperLibrary = this.getById(paperLibraryId);
        PaperLibraryResult paperLibraryResult = new PaperLibraryResult();
        BeanUtils.copyProperties(paperLibrary, paperLibraryResult);
        String fileUrl = paperLibrary.getPath();
        if (StringUtils.isBlank(fileUrl)) {
            throw ExceptionResultEnum.ERROR.exception("未找到图片路径");
        }
        List<JSONObject> objectList = JSON.parseArray(fileUrl, JSONObject.class);
        List<String> fileUrls = new ArrayList<>();
        for (JSONObject jsonObject : objectList) {
            fileUrls.add(paperLibraryCommonService.preViewPath(jsonObject.getString(SystemConstant.PATH), jsonObject.getString(SystemConstant.UPLOAD_TYPE), jsonObject.getString(SystemConstant.TYPE), false));
        }
        PaperScanTaskResult paperScanTaskResult = paperScanTaskService.getByPaperScanTaskId(paperLibrary.getPaperScanTaskId());
        if (paperScanTaskResult != null) {
            paperLibraryResult.setSemesterName(paperScanTaskResult.getSemesterName());
            paperLibraryResult.setPaperArchivesName(paperScanTaskResult.getArchivesName());
            paperLibraryResult.setPaperScanTaskName(paperScanTaskResult.getScanTaskName());
            paperLibraryResult.setCourseName(paperScanTaskResult.getCourseName());
//            paperLibraryResult.setTeacherName(paperScanTaskResult.getTeacherName());
//            paperLibraryResult.setTeachClazzName(paperScanTaskResult.getTeachClazzName());
        }
        paperLibraryResult.setFileUrls(fileUrls);
        paperLibraryResultList.add(paperLibraryResult);
        return paperLibraryResultList;
    }

    @Override
    public List<PaperLibraryResult> toBindPaper(Long paperScanTaskId) {
        Long userId = Long.valueOf(ServletUtil.getRequestHeaderUserId().toString());
        Long schoolId = SystemConstant.convertIdToLong(String.valueOf(ServletUtil.getRequestHeaderSchoolId()));

        // 查询下一个待绑定任务
        List<PaperLibrary> waitPaperLibraryList = this.baseMapper.selectBatchData(paperScanTaskId, userId);
        if (waitPaperLibraryList.isEmpty()) {
            waitPaperLibraryList = createBindData(userId, paperScanTaskId);
        }

        List<PaperLibraryResult> paperLibraryResultList = new ArrayList<>();
        if (!waitPaperLibraryList.isEmpty()) {
            for (PaperLibrary paperLibrary : waitPaperLibraryList) {
                PaperLibraryResult paperLibraryResult = new PaperLibraryResult();
                BeanUtils.copyProperties(paperLibrary, paperLibraryResult);
                String fileUrl = paperLibrary.getPath();
                if (StringUtils.isBlank(fileUrl)) {
                    throw ExceptionResultEnum.ERROR.exception("未找到图片路径");
                }
                List<JSONObject> objectList = JSON.parseArray(fileUrl, JSONObject.class);
                List<String> fileUrls = new ArrayList<>();
                for (JSONObject jsonObject : objectList) {
                    fileUrls.add(paperLibraryCommonService.preViewPath(jsonObject.getString(SystemConstant.PATH), jsonObject.getString(SystemConstant.UPLOAD_TYPE), jsonObject.getString(SystemConstant.TYPE), false));
                }
                PaperScanTaskResult paperScanTaskResult = paperScanTaskService.getByPaperScanTaskId(paperLibrary.getPaperScanTaskId());
                if (paperScanTaskResult != null) {
                    paperLibraryResult.setSemesterName(paperScanTaskResult.getSemesterName());
                    paperLibraryResult.setPaperArchivesName(paperScanTaskResult.getArchivesName());
                    paperLibraryResult.setPaperScanTaskName(paperScanTaskResult.getScanTaskName());
                    paperLibraryResult.setCourseName(paperScanTaskResult.getCourseName());
                }
                paperLibraryResult.setFileUrls(fileUrls);
                paperLibraryResultList.add(paperLibraryResult);
            }
        }
        return paperLibraryResultList;
    }

    @Override
    public IPage<TaskStudentResult> listByStudentCode(String studentCode, Integer pageNumber, Integer pageSize) {
        if (StringUtils.isBlank(studentCode)) {
            throw ExceptionResultEnum.ERROR.exception("请输入考号");
        }
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        return this.baseMapper.listByStudentCode(new Page<>(pageNumber, pageSize), schoolId, studentCode);
    }

    @Override
    public int countScanCount(Long paperScanTaskId) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        QueryWrapper<PaperLibrary> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(PaperLibrary::getSchoolId, schoolId)
                .eq(PaperLibrary::getPaperScanTaskId, paperScanTaskId);
        return this.count(queryWrapper);
    }

    @Override
    public IPage<TaskStudentResult> pageStudent(Long paperScanTaskId, String param, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Page<TaskStudentResult> ipage = new Page<>(pageNumber, pageSize);
        IPage<TaskStudentResult> taskStudentResultIPage = this.baseMapper.listByStudentCode(ipage, schoolId, param);
        for (TaskStudentResult studentResult : taskStudentResultIPage.getRecords()) {
            PaperScanTaskDetail paperScanTaskDetail = paperScanTaskDetailService.getByPaperScanTaskIdAndStudentId(paperScanTaskId, Long.valueOf(studentResult.getStudentId()));
            studentResult.setBindCount(paperScanTaskDetail == null ? 0 : paperScanTaskDetail.getBindCount());
        }
        return taskStudentResultIPage;
    }

    @Override
    public List<String> recognition(Long paperLibraryId, Integer index, Integer x, Integer y, Integer width, Integer height, Integer rotate, RecognitionTypeEnum type) {
        QueryWrapper<PaperLibrary> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(PaperLibrary::getId, paperLibraryId);
        PaperLibrary paperLibrary = this.getOne(queryWrapper);
        List<JSONObject> objectList = JSON.parseArray(paperLibrary.getPath(), JSONObject.class);
        JSONObject jsonObject = objectList.get(index);

        String rootPath = fileStoreUtil.buildPath(UploadFileEnum.DOWNLOAD, true);
        String cutRootPath = fileStoreUtil.buildPath(UploadFileEnum.CUT, true);
        File file;
        try {
            file = paperLibraryCommonService.downloadFile(rootPath, jsonObject.getString(SystemConstant.PATH), jsonObject.getString(SystemConstant.UPLOAD_TYPE), jsonObject.getString(SystemConstant.TYPE));
        } catch (FileNotFoundException e) {
            throw ExceptionResultEnum.ERROR.exception("下载图片失败：" + e.getMessage());
        }
        // 裁剪
        File outFile = new File(cutRootPath, "sub" + file.getName());
        if (!outFile.exists()) {
            outFile.getParentFile().mkdirs();
        }
        File subFile = ImageUtil.subImg(file, x, y, width, height, outFile);
        if (subFile == null) {
            throw ExceptionResultEnum.ERROR.exception("识别区域图片生成失败");
        }

        // 旋转
        File rotateOutFile = new File(cutRootPath, "rotate" + file.getName());
        File rotateFile = ImageUtil.rotate(subFile, rotate, rotateOutFile);

        List<String> stringList = new ArrayList<>();
        try {
            if (RecognitionTypeEnum.OCR.equals(type)) {
                org.json.JSONObject callHandwriting;
                if (rotateFile.exists()) {
                    callHandwriting = ocrUtil.callHandwriting(rotateFile.getPath());
                } else {
                    callHandwriting = ocrUtil.callHandwriting(subFile.getPath());
                }

                // 未联网提示
                if (!callHandwriting.has(SystemConstant.OCR_WORDS_RESULT)) {
                    // ocr错误提示
                    if (callHandwriting.has(SystemConstant.OCR_ERROR_MSG)) {
                        throw ExceptionResultEnum.ERROR.exception(callHandwriting.getString(SystemConstant.OCR_ERROR_MSG));
                    } else {
                        throw ExceptionResultEnum.ERROR.exception("OCR识别异常，请检查网络是否连接或者关闭OCR识别");
                    }
                }

                JSONArray jsonArray = callHandwriting.getJSONArray(SystemConstant.OCR_WORDS_RESULT);
                if (jsonArray.length() == 0) {
                    throw ExceptionResultEnum.ERROR.exception("未正确识别出内容");
                } else {
                    jsonArray.forEach(m -> {
                        org.json.JSONObject object = (org.json.JSONObject) m;
                        if (object.has(SystemConstant.OCR_WORDS)) {
                            stringList.add(object.getString(SystemConstant.OCR_WORDS));
                        }
                    });

                    // 保存识别数据
                    UpdateWrapper<PaperLibrary> updateWrapper = new UpdateWrapper<>();
                    updateWrapper.lambda().set(PaperLibrary::getWordsResult, jsonArray.toString())
                            .set(PaperLibrary::getOcrResult, String.join(",", stringList))
                            .eq(PaperLibrary::getId, paperLibraryId);
                    this.update(updateWrapper);
                }
            } else if (RecognitionTypeEnum.BAR_CODE.equals(type)) {
                String result;
                if (rotateFile.exists()) {
                    result = BarCodeUtils.decode(rotateFile);
                } else {
                    result = BarCodeUtils.decode(subFile);
                }
                if (StringUtils.isNotBlank(result)) {
                    stringList.add(result);
                }
                // 保存识别数据
                UpdateWrapper<PaperLibrary> updateWrapper = new UpdateWrapper<>();
                updateWrapper.lambda().set(PaperLibrary::getWordsResult, result)
                        .eq(PaperLibrary::getId, paperLibraryId);
                this.update(updateWrapper);
            } else {
                throw ExceptionResultEnum.ERROR.exception("不支持的识别方式");
            }
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("识别异常");
        } finally {
            try {
                FileUtils.forceDelete(file);
//                FileUtils.forceDelete(subFile);
//                FileUtils.forceDelete(rotateFile);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return stringList;
    }

    @Transactional
    @Override
    public Boolean abnormal(Long paperLibraryId) {
        PaperLibrary paperLibrary = this.getById(paperLibraryId);
        if (paperLibrary == null) {
            throw ExceptionResultEnum.ERROR.exception("绑定对象有误，图片已删除，请刷新数据再试");
        }

        Long paperScanTaskId = paperLibrary.getPaperScanTaskId();
        PaperScanTask paperScanTask = paperScanTaskService.getById(paperScanTaskId);
        UpdateWrapper<PaperScanTask> taskUpdateWrapper = new UpdateWrapper<>();
        taskUpdateWrapper.lambda().set(PaperScanTask::getAbnormalCount, paperScanTask.getAbnormalCount() + 1)
                .eq(PaperScanTask::getId, paperScanTaskId);
        paperScanTaskService.update(taskUpdateWrapper);

        UpdateWrapper<PaperLibrary> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(PaperLibrary::getAbnormal, true).eq(PaperLibrary::getId, paperLibraryId);
        return this.update(updateWrapper);
    }

    @Override
    public Integer maxSequence(Long paperScanTaskId) {
        return this.baseMapper.maxSequence(paperScanTaskId);
    }

    private List<PaperLibrary> createBindData(Long userId, Long paperScanTaskId) {
        try {
            lockService.waitlock(LockType.BIND_PAPER_TASK, paperScanTaskId);
            PaperScanTask paperScanTask = paperScanTaskService.getById(paperScanTaskId);
            if (paperScanTask.getCheckUserId() != null && !userId.equals(paperScanTask.getCheckUserId())) {
                throw ExceptionResultEnum.ERROR.exception("该任务已被其它人绑定，请刷新数据");
            }

            UpdateWrapper<PaperScanTask> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(PaperScanTask::getCheckUserId, userId)
                    .eq(PaperScanTask::getId, paperScanTaskId);
            paperScanTaskService.update(updateWrapper);

            List<PaperLibrary> paperLibraryList = this.baseMapper.listUnBindData(paperScanTaskId);
            if (!paperLibraryList.isEmpty()) {
                paperLibraryList.forEach(m -> {
                    if (m.getPaperScanTaskDetailId() == null) {
                        m.setUserId(userId);
                    }
                });
                this.updateBatchById(paperLibraryList);
            }
            return paperLibraryList;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            lockService.unlock(LockType.BIND_PAPER_TASK, paperScanTaskId);
        }
        return new ArrayList<>();
    }

}
