package com.qmth.paper.library.common.entity;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.paper.library.common.base.BaseEntity;
import com.qmth.paper.library.common.bean.dto.syssetting.SettingObject;
import com.qmth.paper.library.common.bean.dto.syssetting.SimpleObject;
import com.qmth.paper.library.common.contant.SysSettingConstant;
import io.swagger.annotations.ApiModelProperty;
import org.apache.commons.lang3.StringUtils;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 系统设置表
 * </p>
 */
@TableName("sys_setting")
public class SysSetting extends BaseEntity implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "学校id")
    @JsonSerialize(using = ToStringSerializer.class)
    @TableField("school_id")
    private Long schoolId;


    /**
     * 是否开启全局匹配
     */
    @TableField("parameter_object")
    private String parameterObject;

    public Long getSchoolId() {
        return schoolId;
    }

    public void setSchoolId(Long schoolId) {
        this.schoolId = schoolId;
    }

    public String getParameterObject() {
        return parameterObject;
    }

    public void setParameterObject(String parameterObject) {
        this.parameterObject = parameterObject;
    }

    /**
     * 初始化数据
     */
    public static SysSetting init(Long schoolId) {
        List<SettingObject> settingObjects = new ArrayList<>();
        List<SimpleObject> simpleObjects1 = new ArrayList<>();
        simpleObjects1.add(new SimpleObject(SysSettingConstant.SYS_VERIFICATION_CODE_LOGIN_ENABLE, "验证码登录", "false", 2, true));
        simpleObjects1.add(new SimpleObject(SysSettingConstant.SYS_SMS_NOTIFICATION_ENABLE, "短信提醒功能", "false", 3, true));

        settingObjects.add(new SettingObject("系统设置", 1, simpleObjects1));

        List<SimpleObject> simpleObjects2 = new ArrayList<>();
        simpleObjects2.add(new SimpleObject(SysSettingConstant.OCR_OPEN, "OCR识别", "false", 1, true));
        simpleObjects2.add(new SimpleObject(SysSettingConstant.BAR_CODE_OPEN, "条码识别", "false", 2, true));

        settingObjects.add(new SettingObject("识别设置", 2, simpleObjects2));

        SysSetting sysSetting = new SysSetting();
        sysSetting.setSchoolId(schoolId);
        sysSetting.setParameterObject(JSON.toJSONString(settingObjects));
        return sysSetting;
    }

    public static String parseParameterObject(SysSetting sysSetting, String key) {
        if (sysSetting != null && StringUtils.isNotBlank(sysSetting.getParameterObject())) {
            List<SettingObject> settingObjects = JSON.parseArray(sysSetting.getParameterObject(), SettingObject.class);
            for (SettingObject settingObject : settingObjects) {
                for (SimpleObject simpleObject : settingObject.getData()) {
                    if (simpleObject.getCode().equals(key)) {
                        return simpleObject.getValue();
                    }
                }
            }
        }
        return null;
    }
}
